#ifndef HAL_RUNTIME_H_
#define HAL_RUNTIME_H_

/* Hal runtime structure is hidden but it is based on hal peer API.
 * A user can get only the runtime state, abort the runtime work,
 * send request commands and register callbacks for other reply and request
 * commands.
 */

/* vmachined uses these callbacks. */
struct hal_runtime_callbacks {
	void 	    (*connect_callback)(unsigned int inputs_sz, unsigned int outputs_sz);
	void	    (*cycle_begin_callback)(void);
	void	    (*cycle_run_callback)(void);
	void	    (*cycle_end_callback)(void);
	void	    (*disconnect_callback)(void);

	const char *(*write_inputs_callback)(const char inputs[HAL_IMAGE_MAX+1], unsigned int off);
	const char *(*write_outputs_callback)(const char outputs[HAL_IMAGE_MAX+1], unsigned int off);
	const char *(*read_inputs_callback)(char inputs[HAL_IMAGE_MAX+1], unsigned int len, unsigned int off);
	const char *(*read_outputs_callback)(char outputs[HAL_IMAGE_MAX+1], unsigned int len, unsigned int off);
};

/* Four states are non-waiting. In other states we wait till
 * a timeout or coming to a non-waiting state. */
enum {
	HAL_RT_STATE_CONN,
	HAL_RT_STATE_DISCONN,
	HAL_RT_STATE_ERROR,
	HAL_RT_STATE_DISABLED,
	/* waiting reply states */
	HAL_RT_STATE_CONN_WAIT,
	HAL_RT_STATE_DISCONN_WAIT,
	HAL_RT_STATE_CR_WAIT,
	/* synching states  */
	HAL_RT_STATE_CB_WAIT,
	HAL_RT_STATE_CE_WAIT
};

/* Return the runtime state. */
int	hal_runtime_state();

/* Stop runtime work and set state to HAL_RT_STATE_DISABLED. */
void	hal_runtime_disable();

/* Init some internals structures. */
int	hal_runtime_init(struct io_context *io, int wfd, int rfd, int timeout);

/* Register callback handlers. */
int	hal_runtime_register_callbacks(struct hal_runtime_callbacks *new_cb);

/* Deinit runtime and free resources. */
void	hal_runtime_deinit();

/* Send request commands to the hal manager. */
void	hal_runtime_connect();
void	hal_runtime_cycle_begin();
void	hal_runtime_cycle_run();
void	hal_runtime_cycle_end();
void	hal_runtime_disconnect();

#endif
